#ifndef INCLUDE_AVREGCONF_COMMON_H
#define INCLUDE_AVREGCONF_COMMON_H 1

#include <stdint.h>
#include <limits.h>

/**
 * Положительный результат
 */
#define AVREGPARSE_SUCCESS 1

/**
 * Отрицательный результат, ошибка.
 */
#define AVREGPARSE_FAILED  0

#define AVREGPARSE_TRUE  1
#define AVREGPARSE_FALSE 0

#define IS_STR_EMPTY(s) ( !(s) || (s)[0] == '\0' )

/*
 * Макс. длина + \z имени параметра
 */
#define AVREGCONF_PARAMNAME_SZ 32

#ifndef MAX
#  define MAX(X,Y) ((X)>(Y)?X:Y)
#endif
#ifndef MIN
#  define MIN(X,Y) ((X)<(Y)?X:Y)
#endif

#define MAX_DEVS 1001
#define AVREG_USERNAME_MAXCHARS 65
// for 16 plain chars max, see sha-256 crypt(3)
#define AVREG_PASSENC_MAXCHARS 43

#define timeval_add(result, a, b)  \
   do { \
      (result)->tv_sec  = (a)->tv_sec  + (b)->tv_sec;  \
      (result)->tv_usec = (a)->tv_usec + (b)->tv_usec; \
      if ((result)->tv_usec >= 1000000L) { \
         ++(result)->tv_sec; \
         (result)->tv_usec -= 1000000L; \
      } \
   } while (0)

#define timeval_sub(result, a, b)  \
   do { \
      (result)->tv_sec  = (a)->tv_sec  - (b)->tv_sec;  \
      (result)->tv_usec = (a)->tv_usec - (b)->tv_usec; \
      if ((result)->tv_usec < 0L) {  \
         --(result)->tv_sec; \
         (result)->tv_usec += 1000000L; \
      } \
   } while (0)

#define timespec_add(result, a, b) \
   do {  \
      (result)->tv_sec  = (a)->tv_sec  + (b)->tv_sec;  \
      (result)->tv_nsec = (a)->tv_nsec + (b)->tv_nsec; \
      if ((result)->tv_nsec >= 1000000000L) {  \
         ++(result)->tv_sec; \
         (result)->tv_nsec -= 1000000000L;  \
      }  \
   } while (0)

#define timespec_sub(result, a, b)  \
   do {  \
      (result)->tv_sec  = (a)->tv_sec  - (b)->tv_sec;  \
      (result)->tv_nsec = (a)->tv_nsec - (b)->tv_nsec; \
      if ((result)->tv_nsec < 0L) {  \
         --(result)->tv_sec;  \
         (result)->tv_nsec += 1000000000L; \
      }  \
   } while (0)


#define timespec2val(spec, val) \
   do { \
      (val)->tv_sec  = (spec)->tv_sec;    \
      (val)->tv_usec = (spec)->tv_nsec / 1000L; \
   } while (0)

#define timeval2spec(val, spec) \
   do { \
      (spec)->tv_sec  = (val)->tv_sec;    \
      (spec)->tv_nsec = (val)->tv_usec * 1000L; \
   } while (0)

#define timeval_isset(tvp)	((tvp)->tv_sec || (tvp)->tv_usec)
#define timeval_clear(tvp)	((tvp)->tv_sec = (tvp)->tv_usec = 0L)
#define timeval_cpy(dst, src)	\
   do { \
      (dst)->tv_sec = (src)->tv_sec;   \
      (dst)->tv_usec = (src)->tv_usec; \
   } while (0)

#define timespec_isset(tsp)	((tsp)->tv_sec || (tsp)->tv_nsec)
#define timespec_clear(tsp)	((tsp)->tv_sec = (tsp)->tv_nsec = 0L)
#define timespec_cpy(dst, src) \
   do { \
      (dst)->tv_sec = (src)->tv_sec;   \
      (dst)->tv_nsec = (src)->tv_nsec; \
   } while (0)

#define timespec_add_ms(dst, src, ms) \
   do { \
      (dst)->tv_sec  = (src)->tv_sec  + (ms) / 1000L; \
      (dst)->tv_nsec = (src)->tv_nsec + (((ms)%1000L) * 1000000L); \
      if ( (dst)->tv_nsec >= 1000000000L ) { \
         ++(dst)->tv_sec; \
         (dst)->tv_nsec -= 1000000000L; \
      } \
   } while (0)

#define timespec_add_us(dst, src, us) \
   do { \
      (dst)->tv_sec  = (src)->tv_sec  + (us) / 1000000L; \
      (dst)->tv_nsec = (src)->tv_nsec + (((us)%1000000L) * 1000L); \
      if ( (dst)->tv_nsec >= 1000000000L ) { \
         ++(dst)->tv_sec; \
         (dst)->tv_nsec -= 1000000000L; \
      } \
   } while (0)

#define timespec_sub_ms(dst, ms) \
   do { \
      (dst)->tv_sec  -= (ms) / 1000L; \
      (dst)->tv_nsec -= ((ms)%1000L) * 1000000L; \
      if ( (dst)->tv_nsec < 0L ) { \
         --(dst)->tv_sec; \
         (dst)->tv_nsec += 1000000000L; \
      } \
   } while (0)

#define timespec_sub_us(dst, us) \
   do { \
      (dst)->tv_sec  -= (us) / 1000000L; \
      (dst)->tv_nsec -= ((us)%1000000L) * 1000L; \
      if ( (dst)->tv_nsec < 0L ) { \
         --(dst)->tv_sec; \
         (dst)->tv_nsec += 1000000000L; \
      } \
   } while (0)

#define timeval_sub_ms(dst, ms) \
   do { \
      (dst)->tv_sec  -= (ms) / 1000L; \
      (dst)->tv_usec -= ((ms) % 1000L) * 1000L; \
      if ( (dst)->tv_usec < 0L ) { \
         --(dst)->tv_sec; \
         (dst)->tv_usec += 1000000L; \
      } \
   } while (0)

#define timeval_add_ms(dst, src, ms) \
   do { \
      (dst)->tv_sec  = (src)->tv_sec  + (ms) / 1000L; \
      (dst)->tv_usec = (src)->tv_usec + (((ms)%1000L) * 1000L); \
      if ( (dst)->tv_usec >= 1000000L ) { \
         ++(dst)->tv_sec; \
         (dst)->tv_usec -= 1000000L; \
      } \
   } while (0)

#define timeval_add_us(dst, us) \
   do { \
      (dst)->tv_sec  +=  (us) / 1000000L; \
      (dst)->tv_usec +=  (us) % 1000000L; \
      if ( (dst)->tv_usec > 1000000L ) { \
         ++(dst)->tv_sec; \
         (dst)->tv_usec -= 1000000L; \
      } \
   } while (0)

#define timeval_sub_us(dst, us) \
   do { \
      (dst)->tv_sec  -=  (us) / 1000000L; \
      (dst)->tv_usec -=  (us) % 1000000L; \
      if ( (dst)->tv_usec < 0L ) { \
         --(dst)->tv_sec; \
         (dst)->tv_usec += 1000000L; \
      } \
   } while (0)

// XXX doesn't work with ">=" "<=" CMP
# define timeval_cmp(a, b, CMP) 		   \
   ( ((a)->tv_sec ==  (b)->tv_sec) ?   \
   ( (a)->tv_usec CMP (b)->tv_usec) : 	\
   ( (a)->tv_sec  CMP (b)->tv_sec) )

# define timeval_equal(a, b) ( ((a)->tv_sec ==  (b)->tv_sec) && ((a)->tv_usec ==  (b)->tv_usec) )

#define ms2timespec(ms, ts) \
   do { \
      (ts)->tv_sec  = (ms) / 1000L; \
      (ts)->tv_nsec = ((ms)%1000L) * 1000000L; \
   } while (0)

#define us2timespec(us, ts) \
   do { \
      (ts)->tv_sec  = (us) / 1000000L; \
      (ts)->tv_nsec = ((us)%1000000L) * 1000L; \
   } while (0)

#define us2timeval(us, tv) \
   do { \
      (tv)->tv_sec  = (us) / 1000000L; \
      (tv)->tv_usec = (us) % 1000000L; \
   } while (0)

/* AvregRational */
#ifndef AvregRational
typedef struct AvregRational {
   int num;
   int den;
} AvregRational;
#endif

/* min fps: 1 frame per 5 seconds */
/* max fps: 60 frames per second  */
#define AVREG_FPS_MAX 60
#define AVREG_FPS_R_MIN (AvregRational){1, 5}
#define AVREG_FPS_R_MAX (AvregRational){AVREG_FPS_MAX, 1}

#define avrational_cpy(dst, src) do { \
      (dst)->num = (src)->num;       \
      (dst)->den = (src)->den;       \
   } while (0)

#define avrational_set(r, NUM, DEN) do { \
      (r)->num = NUM;       \
      (r)->den = DEN;       \
   } while (0)

#define AVRATIONAL_EMPTY(r) ( r.num == 0 || r.den == 0 )
#define FPS_TO_FRAME_PERIOD_USEC(fps) (((uint64_t)1000000 * fps.den) / fps.num) /* FIXME неточно и опасно */

#define timeval2usec(tv) (int64_t)((int64_t)tv.tv_sec * 1000000 + tv.tv_usec)
#define timespec2usec(ts) (int64_t)((int64_t)ts.tv_sec * 1000000 + ts.tv_nsec/1000)
#define time2usec(t) (int64_t)((int64_t)t * 1000000)

#define MIN_SEC_FMT "%2.2ld:%2.2ld"
#define MIN_SEC_ARG(tv) ((tv)->tv_sec / 60) % 60, (tv)->tv_sec % 60

#define IS_FLAGS_ON_EVERY(state, flags)  ((state & (flags)) == (flags))
#define IS_FLAGS_OFF_EVERY(state, flags) ((state & (flags)) == 0)

#define IS_FLAGS_ON_ONE(state, flags)  ((state & (flags)) != 0)
#define IS_FLAGS_OFF_ONE(state, flags) ((state & (flags)) != (flags))

typedef enum AVREGCONF_FLAGS_ACTION {
   AVREGCONF_FLAGS_GET = 0,
   AVREGCONF_FLAGS_INIT,
   AVREGCONF_FLAGS_RESET,
   AVREGCONF_FLAGS_SET
} AVREGCONF_FLAGS_ACTION;

static inline uint32_t
avregconf_u32flags_diff(uint32_t cur, uint32_t updates, AVREGCONF_FLAGS_ACTION action)
{
   uint32_t diff, s;

   switch (action) {
      case AVREGCONF_FLAGS_SET:
         s = cur;
         break;
      case AVREGCONF_FLAGS_RESET:
         s = ~cur;
         break;
      default: // as GET
         return (cur & updates);
   }

   diff = (updates & s) ^ updates;

   return diff;
} // avregconf_u32flags_diff()

int avregconf_get_line_max(void);

/**
 * Compare two rationals.
 *
 * @param a First rational
 * @param b Second rational
 *
 * @return One of the following values:
 *         - 0 if `a == b`
 *         - 1 if `a > b`
 *         - -1 if `a < b`
 *         - `INT_MIN` if one of the values is of the form `0 / 0`
 */
static inline int avregconf_cmp_q(AvregRational a, AvregRational b) {
    const int64_t tmp = a.num * (int64_t)b.den - b.num * (int64_t)a.den;

    if(tmp) return (int)((tmp ^ a.den ^ b.den)>>63)|1;
    else if(b.den && a.den) return 0;
    else if(a.num && b.num) return (a.num>>31) - (b.num>>31);
    else                    return INT_MIN;
}


#endif
