#ifndef INCLUDE_AVREGCONF_LOG_H
#define INCLUDE_AVREGCONF_LOG_H 1

#include <stdio.h>
#include <stdlib.h> // abort()
#include <syslog.h> // level = syslog's AVREGLOG_ERR, etc ...
#include <stdint.h>

#include <avregconf/common.h>

#define AVREGLOG_STRERROR_R_DECLARE()   \
      char __err_str__[200], *__errp__; \
      int __orig_err__

#define AVREGLOG_STRERROR_R(__errno__) do {                               \
   __orig_err__ = __errno__;                                              \
   __err_str__[0] = '\0';                                                 \
   __errp__ = strerror_r(__orig_err__, __err_str__, sizeof(__err_str__)); \
} while (0)

#define AVREGLOG_ERRNO_N_STR __orig_err__, __errp__

#define AVREGLOG_FAILED_WITH_ERRSTR_FMT " failed with #%d(%s)"
#define AVREGLOG_FAILED_WITH_ERRNO_FMT " failed with #%d"

#define AVREGLOG_CONTEXT_NAME_SZ 50
#define AVREGLOG_STR_SZ 512
#define AVREGLOG_RTIME_FMT_DEF "%b %d %T"
#define AVREGLOG_RELTIME_FMT_DEF "%08.2f"
#define AVREGLOG_FIELDS_SEP_DEF ": "
#define MAX_LC_DEPTH 5

#define AVREGLOG_TIME_INVAL ((int64_t)UINT64_C(0x8000000000000000))

/*
 * #define AVREGLOGPERR(X) avreglog(NULL, LOG_ERR,  "error in %s:%u %s(): %s failed with #%d(%s)", \
 * __FILE__, __LINE__, (const char *)__func__, X, errno, strerror(errno))
 * #define AVREGLOGPERR2(level, fmt, ...) avreglog(NULL, level, fmt" failed with errno #%d(%s)", __VA_ARGS__, errno, strerror(errno))
 * #define AVREGLOGERR3(lc, fmt, ...) avreglog(lc, LOG_ERR, "error in %s:%u %s(): "fmt, __FILE__, __LINE__, (const char *)__func__, __VA_ARGS__ )
 * #define AVREGLOGPERR3(lc, level, fmt, ...) avreglog(lc, level, fmt" failed with errno #%d(%s)", __VA_ARGS__, errno, strerror(errno))
*/

#define AVREGLOGPERR(fmt) do {                                                         \
   AVREGLOG_STRERROR_R_DECLARE();                                                      \
   AVREGLOG_STRERROR_R(errno);                                                         \
   avreglog_r(NULL, LOG_ERR,  "error in %s:%u %s(): %s failed with #%d(%s)",           \
      __FILE__, __LINE__, (const char *)__func__, fmt, AVREGLOG_ERRNO_N_STR);          \
} while (0)

#define AVREGLOGPERR_LC(lc, level, fmt, ...) do {                                      \
   AVREGLOG_STRERROR_R_DECLARE();                                                      \
   AVREGLOG_STRERROR_R(errno);                                                         \
   avreglog_r((lc), level,                                                             \
      fmt " failed with #%d(%s)",                                                      \
      __VA_ARGS__, AVREGLOG_ERRNO_N_STR);                                              \
} while (0)

#ifdef NIK_DEBUG
#  ifndef AVREGLOGTEST
#     define AVREGLOGTEST(fmt, ...) avreglog_r(NULL, LOG_NOTICE,  "%s:%u %s(): +++ "fmt, __FILE__, __LINE__, (const char *)__func__, __VA_ARGS__ )
#  endif
#  ifndef DDD
#     define DDD avreglog_r(NULL, LOG_NOTICE,  "%s:%u %s(): +++++ ===== +++++ ====== ++++++ ======", __FILE__, __LINE__, (const char *)__func__)
#  endif
#else
#  ifndef AVREGLOGTEST
#     define AVREGLOGTEST(fmt, ...)
#  endif
#  ifndef DDD
#     define DDD
#  endif
#endif

typedef enum AVREGLOG_FLAGS {
   AVREGLOG_FLAG_USE_SYSLOG       = 0x0001,
#define AVREGLOG_FLAG_USE_SYSLOG AVREGLOG_FLAG_USE_SYSLOG

   AVREGLOG_FLAG_SKIP_REPEATED    = 0x0002,
#define AVREGLOG_FLAG_SKIP_REPEATED AVREGLOG_FLAG_SKIP_REPEATED

   AVREGLOG_FLAG_PRINT_TIMESTAMP  = 0x0020,
#define AVREGLOG_FLAG_PRINT_TIMESTAMP AVREGLOG_FLAG_PRINT_TIMESTAMP
   AVREGLOG_FLAG_PRINT_TIME_SINCE = 0x0040
#define AVREGLOG_FLAG_PRINT_TIME_SINCE AVREGLOG_FLAG_PRINT_TIME_SINCE
} AVREGLOG_FLAGS;


typedef struct avreglog_ExtraFlags {
   AVREGCONF_FLAGS_ACTION action;
   uint32_t value;
} avreglog_ExtraFlags;

typedef struct avreglog_context_s avreglog_context_t;
struct avreglog_context_s {
   char __name[AVREGLOG_CONTEXT_NAME_SZ];
   const avreglog_context_t *__parent;

   int __level;                        ///< ignored if less 0
   avreglog_ExtraFlags __extra_flags;  ///< ignored if extra_flags.action === AVREGCONF_FLAGS_GET(0)
};

#define SAFE_LC(cnt) (cnt ? &(cnt)->lc : NULL)

uint32_t avreglog_update_flags(AVREGCONF_FLAGS_ACTION action, uint32_t flags);
void avreglog_update_flags_r(AVREGCONF_FLAGS_ACTION action, uint32_t flags, uint32_t *new_flags);
uint32_t avreglog_get_flags(void);
void avreglog_get_flags_r(uint32_t *flags);

void avreglog_update_time(int64_t realtime_us, int64_t monotonic_us);
void avreglog_update_time_r(int64_t realtime_us, int64_t monotonic_us);

/***
 * use syslog LOG_XXX
 * default is LOG_NOTICE
 */
void avreglog_level_set(int level);
int  avreglog_level_get(void);

void avreglog_set_file_targets(FILE *out, FILE *err);

const char *avreglog_context_get_name(const avreglog_context_t *lc);
void avreglog_context_get_name_r(const avreglog_context_t *lc, char *name, size_t name_sz);

#ifdef __GNUC__
void avreglog_set_context(avreglog_context_t *lc, const avreglog_context_t *parent,
      int level,
      avreglog_ExtraFlags *extra_flags,
      const char *name_fmt, ... ) __attribute__ ((__format__ (__printf__, 5, 6)));
void avreglog_context_set_name(avreglog_context_t *lc, const char *name_fmt, ... )
   __attribute__ ((__format__ (__printf__, 2, 3)));
#else
void avreglog_set_context(avreglog_context_t *lc, const avreglog_context_t *parent,
      int level,
      avreglog_ExtraFlags *extra_flags,
      const char *name_fmt, ... );
void avreglog_context_set_name(avreglog_context_t *lc, const char *name_fmt, ... );
#endif

#ifdef __GNUC__
void avreglog_set_context_r(avreglog_context_t *lc, const avreglog_context_t *parent,
      int level,
      avreglog_ExtraFlags *extra_flags,
      const char *name_fmt, ... ) __attribute__ ((__format__ (__printf__, 5, 6)));
void avreglog_context_set_name_r(avreglog_context_t *lc, const char *name_fmt, ... )
   __attribute__ ((__format__ (__printf__, 2, 3)));
#else
void avreglog_set_context_r(avreglog_context_t *lc, const avreglog_context_t *parent,
      int level,
      avreglog_ExtraFlags *extra_flags,
      const char *name_fmt, ... );
void avreglog_context_set_name_r(avreglog_context_t *lc, const char *name_fmt, ... );
#endif

void avreglog_set_proto_context(avreglog_context_t *lc, const avreglog_context_t *parent,
      int level,
      avreglog_ExtraFlags *extra_flags,
      const char *proto_str,
      int has_video, int has_audio,
      int is_output);

void avreglog_set_proto_context_r(avreglog_context_t *lc, const avreglog_context_t *parent,
      int level,
      avreglog_ExtraFlags *extra_flags,
      const char *proto_str,
      int has_video, int has_audio,
      int is_output);

#ifdef __GNUC__
void avreglog(const avreglog_context_t *lc, int level, const char *fmt, ... )  __attribute__ ((__format__ (__printf__, 3, 4)));
#else
void avreglog(const avreglog_context_t *lc, int level, const char *fmt, ... );
#endif


#ifdef __GNUC__
void avreglog_r(const avreglog_context_t *lc, int level, const char *fmt, ... )  __attribute__ ((__format__ (__printf__, 3, 4)));
#else
void avreglog_r(const avreglog_context_t *lc, int level, const char *fmt, ... );
#endif


/**
 * assert() equivalent, that is always enabled.
 */
#ifdef NDEBUG
#  define avreg_assert(cond) do {                                                                \
   if ( !(cond) ) {                                                                              \
      avreglog(NULL, LOG_CRIT, "assertion (%s) failed at %s:%d, send us bugreport, please", \
                              #cond, __FILE__, __LINE__);                                        \
   }                                                                                             \
} while (0)
#else
#  define avreg_assert(cond) do {                                                                \
   if ( !(cond) ) {                                                                              \
      avreglog(NULL, LOG_CRIT, "assertion (%s) failed at %s:%d, send us bugreport, please", \
                              #cond, __FILE__, __LINE__);                                        \
      abort();                                                                                   \
   }                                                                                             \
} while (0)
#endif

#endif // INCLUDE_AVREGCONF_LOG_H
