#ifndef _INCLUDE_AVREGCONF_STR_H_
#define _INCLUDE_AVREGCONF_STR_H_ 1

#include <string.h>
#include <strings.h>

#include <avregconf/common.h>
#include <avregconf/log.h>

#define STR_BEGINS_AT(STR,PREFIX) (STR == strstr(STR,PREFIX)) 
#define STR_BEGINS_CASE_AT(STR,PREFIX) (STR == strcasestr(STR,PREFIX))

#define EQUAL_S(S1,S2)      (0 == avregconf_strcmp(S1, S2, 0, 0))
#define EQUAL_CASE_S(S1,S2) (0 == avregconf_strcmp(S1, S2, 0, 1))

/* строковые ф-ции, используемые при парсинге внутри библиотеки
 * которые могут быть полезными и в приложениях */
#define charSPACE ' '
#define charTAB '\t'
#define charLF '\n'
#define charCR '\r'
#define avregconf_is_digit_char(c) ( '0' <= c && c <= '9' )
#define avregconf_is_space_char(c) ( c == charSPACE || c == charTAB || c == charLF || c == charCR )

static inline int
avregconf_char2hex(char c)
{
	if ('0' <= c && c <= 'f') {
		if (c <= '9') return c - '0';
		if ('A' <= c) {
			if (c <= 'F') return c - 'A' + 10;
			if ('a' <= c) return c - 'a' + 10;
		}
	}
	return -1;
}

static inline int
avregconf_is_alnum_char(char c)
{
	if ('0' <= c) {
		if (c <= '9') return 1;
		if ('A' <= c) {
			if (c <= 'Z') return 1;
			if ('a' <= c && c <= 'z') return 1;
		}
	}
	return 0;
}


static inline int
avregconf_is_safe_char(char c)
{
	if (',' <= c) {
		if (c <= ';') return 1;
		if ('A' <= c) {
			if (c <= 'Z') return 1;
			if ('^' <= c && c <= 'z') return 1;
		}
	}
	return 1;
}


static inline int
avregconf_char_in(const char c, const char *accept)
{
   if ( !accept || accept[0] == '\0' )
      return 0;

    char *p = (char *)accept;
    while ( *p != '\0' ) {
       if ( *p == c ) {
          return 1;
       }
       p++;
    }
    return 0;
} /* char_in() */


static inline char *
avregconf_fetch2newline(const char *abzac)
{
   const char *p = abzac;

   for (;;) {
      if ( *p == '\0' || *p == charCR || *p == charLF) {
         break;
      }
      p++;
   }

   if (*p == '\0') { 
      return (char *)p;
   }

   while ((*p != '\0') && ((*p == charCR) || (*p == charLF))) {
      p++;
   }

   return (char *)p;
}   /* fetch_to_newline() */

static inline char *
avregconf_fetch2newlineORboundary(const char *abzac, const char *boundary, size_t boundary_sz)
{
   const char *p = abzac;
   char *bg;

   if ( *p && ( NULL != ( bg = strstr(p, boundary ))) ) {
      return bg;
   }

   for (;;) {
      if ( *p == 0 || *p == charCR || *p == charLF ) {
         break;
      }
      p++;
   }

   if (*p == 0) {
      return (char *)p;
   }

   while ((*p != 0) && ((*p == charCR) || (*p == charLF))) {
      p++;
   }

   return (char *)p;
}   /* fetch_to_newline() */

/* [A-Za-z0-9].* */
static inline char *
avregconf_fetch2word(const char *line)
{
    const char *p = line;
    while ( *p && !( \
             (*p >= '0' && *p <= '9') /* [0-9] */ || \
             (*p >= 'A' && *p <= 'Z') /* [A-Z] */ || \
             (*p >= 'a' && *p <= 'z') /* [a-z] */ || \
             (*p == '+' || *p == '-') /*  +-   */ )) {
       p++;
    }
    return (char *)p;
}

static inline char *
avregconf_fetch2chars(const char *line, const char stop[])
{
    const char *p = line;
    while ( *p && !avregconf_char_in(*p, stop) ) {
       p++;
    }
    return (char *)p;
}

static inline char *
avregconf_strchrs(const char *line, const char stop[])
{
    const char *p = line;
    for(;;) {
       if (*p == '\0') {
          return NULL;
       }

       if (avregconf_char_in(*p, stop)) {
          return (char *)p;
       }

       p++;
    }
    return NULL; // never
}

static inline char *
avregconf_skip_NULLs(const char *s, int s_sz)
{
   const char *p = s;

   for ( ; s_sz > 0 && *p == '\0'; s_sz-- ) {
      p++;
   }
   return (char *)p;
}

char *avregconf_skip_spaces(const char *line);
char *avregconf_skipr_spaces(const char *line);
int   avregconf_is_empty_str(const char *str);

#define AVREGCONF_TRIML_REJCHARS " \t\r"
#define AVREGCONF_TRIMR_REJCHARS " \t\r\n"
char *avregconf_trim_l  (const char *s, size_t s_sz, const char *reject_chars);
char *avregconf_trim_lz (const char *sz, const char *reject_chars);
char *avregconf_trim_r  (const char *p, char *endp, const char *reject_chars);
char *avregconf_trim_rz (const char *sz, const char *reject_chars);
char *avregconf_explode (const char *src, const char *delim,
                                       const char *spaces_l, const char *spaces_r,
                                       char **begin, char **end);
char *avregconf_fetch_to(const char *p, const char *endp, const char *find);

/* set errno EINVAL or ERANGE on error and return 0 */
long int avregconf_strtol(const char *nptr, int base, long int min, long int max);

/* return 1 - success, 0 - error */
int avregconf_strtol2(const char *str, int base, long int *result);

char *avregconf_strncpy(char *dest, size_t dest_sz, const char *src, size_t src_len);
int avregconf_strcmp(const char *s1, const char *s2, size_t n, int casecmp);
char *avregconf_strtolower_ascii(char *dest, const char *src, size_t n);

/* parse "X:Y" or "XxY" or "X/Y" and return AvregRational { num, den}
 * if !strictly_mode  allowed simple "X" string as AvregRational {X/1}
 * if min,max .den != 0 -> do range checking
 * return 0 on error */
int avregconf_get_q(const char *str,
                int strictly_mode,
                AvregRational min, AvregRational max,
                AvregRational *r);

int avregconf_check_zterm_param_len(const avreglog_context_t *lc, const char *parname, const char *value, size_t max_len);
#endif // _INCLUDE_AVREGCONF_STR_H_
