/*
 * tcp.h
 *
 *   (c) 2011 Andrew Nikitin <nik-a@mail.ru>
 *
 */
#ifndef _INCLUDE_AVREGTCP_H_
#define _INCLUDE_AVREGTCP_H_

#include <sys/types.h>
#include <netinet/in.h>


#define MAX_IP_PER_NODE 20

/***
 * Get binary address of UP interface or IP string or hostname.
 *
 * @param[in]  node        Name of local interface or IP string or hostname
 * @param[in]  port        Port
 * @paramp[in] bind_to     Bind to this IPv4 address.
 * @param[in]  bind_to6    Bind to this IPv6 address.
 * @param[in,out] in_progress If not NULL, O_NONBLOCK flag will be used.
 *                            and return 1 if connect() return error with
 *                            EINPROGRESS errno.
 * @param[out] errsrc      Buffer to copy error message.
 * @param[in]  errsrc_sz   Size of errsrc buffer.
 * @return 1 - found, 0 - not.
 *
 */
int tcp_connect(const char *node, unsigned short port,
                const struct in_addr  *bind_to,
                const struct in6_addr *bind_to6,
                int *in_progress,
                char *errsrc, size_t errsrc_sz);

#endif // _INCLUDE_AVREGTCP_H_
