#ifndef INCLUDE_AVREGCONF_AVREG_USERS_H
#define INCLUDE_AVREGCONF_AVREG_USERS_H 1

#include <stdint.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/ip.h> /* superset of previous */

#include "avregconf/common.h"
#include "avregconf/dyn-list.h"

// in minutes
#define FORCED_SAVING_LIMIT_MIN    0 // unlim
#define FORCED_SAVING_LIMIT_DEF    5
#define FORCED_SAVING_LIMIT_MAX 4320 // 3 hour

/**
 * Структура для хранения данных пользовательской учётки.
 */
typedef struct avreg_user_s {
    char     ip_acl[512];      ///< Строка - разрешённый хост, например 192.168.1.0/255.255.255.0 */
    struct   in_addr ip4_a;
    struct   in_addr ip4_m;

    char     login[AVREG_USERNAME_MAXCHARS + 1];   ///< логин
    char     encoded_passwd[AVREG_PASSENC_MAXCHARS + 1];  ///< пароль

    int      status; ///< Авреговая группа: devel(0) install(1) admin(2) arch(3) operator(4) viewer(5)

    char     allow_cams[512];
    int      allow_cams_bin[MAX_DEVS]; ///< Массив номеров "разрешённых" камер.
    unsigned int allow_cams_bin_nbr;

    int      forced_saving_limit; ///< Максимальная длительность принудительной записи (по команде пользователя) в минутах.

    int      sessions_per_user; ///< Ограничение количества одновременных просмотров (камер суммарно).

    AvregRational limit_fps;      ///< Ограничение скорости отдаваемого видеопотока для каждой камеры
    AvregRational nonmotion_fps;  ///< --//-- ограничение потока "спокойных" кадров в отсутствии движения.
    int      limit_kbps;          ///< в kb per second (не реализовано) 

    time_t   session_time_sec; ///< Ограничения длительности одного непрерывного сеанса просмотра камеры в реальном времени.
    uint64_t session_volume_b; ///< Ограничение сеанса по "закаченному" объёму, в МегаБайтах
} avreg_user_t;


typedef void   (* avreg_user_remove_cbk_t) (avreg_user_t *au);

/***
 * Создаёт список DYN_LIST
 * Для сравнения используется avreg_user_comp() сравнивающая логины и IPv4 или адреса подсетей
 * с помощью avregconf_ipv4_cmp().
 * Если в dlst_flags DLST_FLAG_CREATE_MUTEX, avreg_add_user_r() будет использовать мьютекс,
 * блокирующий список на время добавления.
 */
DYN_LIST *avreg_users_list_create(int dlst_flags, avreg_user_remove_cbk_t avreg_user_remove_cbk);

/***
 * Cравнивает логины и IPv4 или адреса подсетей с помощью avregconf_ipv4_cmp()
 */
int avreg_user_comp(const avreg_user_t *au1, const avreg_user_t *au2);

/***
 * Освобождает DYN_LIST список пользователей.
 */
void avreg_users_list_free(DYN_LIST *avreg_users);

/**
 * Добавляет учётную запись пользователя список.
 */
int
avreg_add_user_r(DYN_LIST *avreg_users,
               const char *ip_acl,
               const char *name,
               const char *passwd_encoded,
               int status,
               const char *allow_cams,
               int forced_saving_limit,
               int sessions_per_user,
               const char *limit_fps,
               const char *nonmotion_fps,
               int limit_kbps,
               int session_time,
               int session_volume);

char *avreg_user2str(avreg_user_t *au);
char *avreg_user2str_r(char *str, size_t max, avreg_user_t *au);

int avreg_check_passwd(const char *encrypted_password, const char *plain_password);
void avreg_user_clear(avreg_user_t *au);


#endif
